/*
 * (C) Dhananjay Nene - 2005
 * This source is licensed under Creative Commons License (Attribution 2.0)
 * (http://creativecommons.org/licenses/by/2.0/legalcode)
 */

package com.dnene.ditutorial.nocontainer;

import java.io.InputStream;
import java.lang.reflect.Constructor;
import java.util.Properties;

import com.dnene.ditutorial.common.AirlineAgency;
import com.dnene.ditutorial.common.CabAgency;
import com.dnene.ditutorial.common.TripPlanner;
import com.dnene.ditutorial.harness.AbstractHarness;


/**
 * An implementation of the execution harness without using
 * any DI container
 * 
 * @author Dhananjay Nene
 */
public class NoContainerHarness extends AbstractHarness
{
	
	public static void main(String[] args) throws Exception
	{
		new NoContainerHarness().runHarness();	
	}

	/* (non-Javadoc)
	 * @see com.dnene.ditutorial.harness.Harness#configure()
	 */
	public void configure() throws Exception
	{
		// No activities here - since there is no container 
		// and therefore no container specific configuration 
		// required.
	}

	/**
	 * Gets a reference to the trip planner.
	 * 
	 * Note that a TripPlanner depends on having references to
	 * both an AirlineAgency and a CabAgency
	 * 
	 * In this case there is no container to support lookups or perform
	 * registration of the various services. As a result we
	 * need to build custom mechanisms to get a reference to the
	 * desired services, and resolved the dependencies within them
	 * as well.
	 * 
	 * In the scenario coded below, the service classes are looked up
	 * from a property file, and reflection is used to instantiate the
	 * services. Any dependencies necessary for the services are
	 * resolved by passing in the necessary references in the 
	 * constructor parameters.
	 * 
	 * @see com.dnene.ditutorial.harness.Harness#getPlanner()
	 */
	public TripPlanner getPlanner() throws Exception
	{
		AirlineAgency airlineAgency = null;
		CabAgency cabAgency = null;
		TripPlanner tripPlanner = null;

		// Get class names from property file
		InputStream propertyFile = getClass().getClassLoader().getResourceAsStream("nocontainer-agency.properties");
		Properties properties = new Properties();
		properties.load(propertyFile);
		
		Class airlineAgencyClass = Class.forName(properties.getProperty("airline-agency-class"));
		Class cabAgencyClass = Class.forName(properties.getProperty("cab-agency-class"));
		Class tripPlannerClass = Class.forName(properties.getProperty("trip-planner-class"));
		
		if (AirlineAgency.class.isAssignableFrom(airlineAgencyClass))
		{
			// Instantiate airline agency
			airlineAgency = (AirlineAgency) airlineAgencyClass.newInstance();
		}
		if (CabAgency.class.isAssignableFrom(cabAgencyClass))
		{
			// Instantiate CabAgency, and satisfy its dependency on an airlineagency.
			
			Constructor constructor = cabAgencyClass.getConstructor(new Class[]{AirlineAgency.class});
			cabAgency = (CabAgency) constructor.newInstance(new Object[]{airlineAgency});
		}	
		if (TripPlanner.class.isAssignableFrom(tripPlannerClass))
		{
			Constructor constructor = tripPlannerClass.getConstructor(new Class[]{AirlineAgency.class,CabAgency.class});
			tripPlanner = (TripPlanner) constructor.newInstance(new Object[]{airlineAgency,cabAgency});
		}
		return tripPlanner;
	}

}
